@@ -1,12 +1,12 @@
-import unittest
-import responses
 import json
-import tempfile
 import os
+import tempfile
+import unittest
 
+import responses
+
+from slims.internal import Attachment, Record
 from slims.slims import Slims
-from slims.slims import Attachment
-from slims.slims import Record
 
 
 class Test_Attachments(unittest.TestCase):
